var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import React, { useEffect } from 'react';
import { createEditor, Editor, Node, Transforms } from 'slate';
import { withHistory } from 'slate-history';
import { DefaultElement, Editable, ReactEditor, Slate, withReact } from 'slate-react';
import { SmartInputFetcher } from './SmartInputFetcher';
export function SmartInputDev(props) {
    var editor = React.useMemo(function () { return withHistory(withReact(createEditor())); }, []);
    var _a = React.useState(null), completion = _a[0], setCompletion = _a[1];
    var fetcher = React.useState(new SmartInputFetcher(function (text, completion) {
        addCompletion(completion);
    }, props.modelId, props.modelId ? props.mockCompletion : function () { return ' -- sample completion --'; }))[0];
    var acceptCompletion = function () {
        EditorHelper.acceptCompletion(editor, fetcher, completion);
        setCompletion(null);
    };
    var addCompletion = function (completion) {
        if (props.value) {
            EditorHelper.addCompletion(editor, fetcher, completion);
            setCompletion(completion);
        }
    };
    var clearCompletion = function () {
        EditorHelper.removeCompletion(editor);
        setCompletion(null);
    };
    fetcher.addCompletion = function (text, completion) {
        var currText = props.value;
        if (completion && (text + completion).startsWith(currText)) {
            addCompletion(completion);
        }
    };
    var initialValue = [
        {
            type: 'text',
            children: [{ text: props.value, type: undefined }],
        },
    ];
    useEffect(function () {
        if (props.editorRef) {
            props.editorRef.current = __assign(__assign({}, ReactEditor.toDOMNode(editor, editor)), { focus: function () {
                    setTimeout(function () {
                        ReactEditor.focus(editor);
                        if (props.value) {
                            Transforms.select(editor, Editor.end(editor, []));
                        }
                        else {
                            Transforms.select(editor, Editor.start(editor, []));
                        }
                    }, 50);
                } });
        }
    });
    var renderElement = React.useCallback(function (p) {
        switch (p.element.type) {
            case 'text':
                return props.renderText(__assign({}, p));
            case 'completion':
                return props.renderCompletion(__assign(__assign({}, p), { contentEditable: 'false', suppressContentEditableWarning: true }));
            default:
                return React.createElement(DefaultElement, __assign({}, p));
        }
    }, [props.renderText, props.renderCompletion]);
    var renderPlaceholder = React.useCallback(function (p) {
        return props.renderPlaceholder(__assign(__assign(__assign({}, p), (p.attributes || {})), { style: __assign({ position: 'absolute', pointerEvents: 'none', width: '100%', maxWidth: '100%', userSelect: 'none', textDecoration: 'none' }, (p.style || {})) }));
    }, [props.renderPlaceholder]);
    return (React.createElement(Slate, { editor: editor, value: initialValue, onChange: function (value) {
            var _a;
            var text = EditorHelper.serialize(value.filter(function (n) { return n.type !== 'completion'; }));
            (_a = fetcher.fetchCompletion(text, props.context || {})) === null || _a === void 0 ? void 0 : _a.catch(function (e) { return console.error(e); });
            fetcher.sendEvents(text, props.context || {});
            props.onChange(text);
        } },
        React.createElement(Editable, { placeholder: props.placeholder, className: props.className, style: __assign(__assign({}, (props.style || {})), { whiteSpace: props.multiline ? undefined : 'pre' }), onPaste: function (event) {
                clearCompletion();
                event.preventDefault();
                var value = event.clipboardData.getData('Text');
                if (!props.multiline) {
                    editor.insertText(value.replaceAll('\n', ''));
                }
                else {
                    editor.insertText(value);
                }
            }, renderPlaceholder: props.renderPlaceholder ? renderPlaceholder : undefined, renderElement: renderElement, onBlur: function (e) {
                clearCompletion();
                fetcher.currentCompletion = null;
                if (props.onBlur) {
                    props.onBlur(e);
                }
            }, onFocus: function (e) {
                if (props.onFocus) {
                    props.onFocus(e);
                    if (editor.selection == null) {
                        if (props.value) {
                            Transforms.select(editor, Editor.end(editor, []));
                        }
                        else {
                            Transforms.select(editor, Editor.start(editor, []));
                        }
                    }
                }
            }, onKeyDown: function (event) {
                var _a, _b, _c;
                var hasCompletion = Boolean((_b = (_a = completion === null || completion === void 0 ? void 0 : completion.completion) === null || _a === void 0 ? void 0 : _a.trim()) === null || _b === void 0 ? void 0 : _b.length);
                if (event.key === 'Tab' && hasCompletion) {
                    event.preventDefault();
                    acceptCompletion();
                    fetcher.currentCompletion = null;
                }
                else if (event.key === 'ArrowRight' && hasCompletion) {
                    event.preventDefault();
                    acceptCompletion();
                    fetcher.currentCompletion = null;
                }
                else if (event.key === 'ArrowLeft' && hasCompletion) {
                    clearCompletion();
                    fetcher.currentCompletion = null;
                }
                else if ((event.key === 'ArrowDown' || event.key === 'ArrowUp') && hasCompletion) {
                    clearCompletion();
                }
                else if (event.key === 'Backspace') {
                    fetcher.currentCompletion = null;
                    clearCompletion();
                }
                else if (event.key === 'Enter' && !event.metaKey) {
                    event.preventDefault();
                    if (props.multiline) {
                        clearCompletion();
                        editor.insertText('\n');
                    }
                }
                else if (event.key !== 'Shift') {
                    if (completion &&
                        !event.metaKey &&
                        ((_c = completion.completion) === null || _c === void 0 ? void 0 : _c.startsWith(event.key)) &&
                        event.key.length === 1) {
                        var newCompletionText = completion.completion.substring(1);
                        EditorHelper.updateCompletion(editor, newCompletionText);
                        setCompletion(__assign(__assign({}, completion), { completion: newCompletionText }));
                    }
                    else {
                        fetcher.currentCompletion = null;
                        clearCompletion();
                    }
                }
                if (props.onKeyDown) {
                    props.onKeyDown(event);
                }
            } })));
}
var EditorHelper = {
    isCompletionActive: function (editor) {
        var completion = editor.children.find(function (n) { return n.type === 'completion'; });
        return !!completion;
    },
    isCursorAtEndOfDocument: function (editor) {
        var sel = editor.selection;
        if (!sel) {
            return false;
        }
        var node = sel.anchor.path[0];
        var offset = sel.anchor.offset;
        if (node !== editor.children.length - 1) {
            return false;
        }
        return offset === Node.string(editor.children[node]).length;
    },
    acceptCompletion: function (editor, fetcher, completion) {
        var isActive = EditorHelper.isCompletionActive(editor);
        if (isActive && (completion === null || completion === void 0 ? void 0 : completion.completion)) {
            EditorHelper.removeCompletion(editor);
            editor.insertText(completion.completion);
            fetcher.updateCompletion(completion.sid, { accepted: true });
        }
    },
    addCompletion: function (editor, fetcher, completion) {
        var isActive = EditorHelper.isCompletionActive(editor);
        var isCursorEnd = EditorHelper.isCursorAtEndOfDocument(editor);
        if (!isActive && isCursorEnd && (completion === null || completion === void 0 ? void 0 : completion.completion)) {
            Transforms.insertNodes(editor, {
                type: 'completion',
                children: [{ text: completion.completion }],
            }, {
                at: [editor.children.length],
            });
            fetcher.updateCompletion(completion.sid, { seen: true });
        }
    },
    removeCompletion: function (editor) {
        var isActive = EditorHelper.isCompletionActive(editor);
        if (isActive) {
            Transforms.removeNodes(editor, {
                at: [editor.children.length - 1],
            });
        }
    },
    updateCompletion: function (editor, newCompletion) {
        var isActive = EditorHelper.isCompletionActive(editor);
        if (isActive && newCompletion.length > 0) {
            Transforms.delete(editor, {
                at: {
                    anchor: { path: [editor.children.length - 1, 0], offset: 0 },
                    focus: { path: [editor.children.length - 1, 0], offset: 1 },
                },
            });
        }
    },
    serialize: function (nodes) {
        return nodes.map(function (n) { return Node.string(n); }).join('\n');
    },
};
//# sourceMappingURL=SmartInput.js.map